#include <algorithm>
#include <numeric>
#include <print>
#include <ranges>
#include <string>
#include <vector>

bool check_luhn(const std::vector<int>& number)
{
    auto view = number
        | std::views::reverse
        | std::views::drop(1)
        | std::views::chunk(2)
        | std::views::transform([](auto digit_pair) -> int {
            int sum {2 * digit_pair.front()};
            if (sum > 9)
                sum -= 9;
            if (digit_pair.size() == 2) // Add odd digit if it exists
                sum += digit_pair.back();
            return sum;
        });
    const int sum = std::reduce(begin(view), end(view));
    const int check_digit_calculated = (10 - (sum % 10)) % 10;
    const int check_digit = number.back();
    return check_digit_calculated == check_digit;
}

int main()
{
    /*** Please do not save your real credit card information in this source file. ***/
    const std::vector number {2, 4, 4, 5, 3, 9, 4, 2, 5, 8, 8, 1, 1, 3, 6, 9};

    if (check_luhn(number))
        std::println("Validation succeeded.");
    else
        std::println("Validation failed.");
}