#include <algorithm>
#include <fstream>
#include <print>
#include <ranges>
#include <sstream>
#include <string>
#include <unordered_map>
#include <utility>
#include <vector>

/* Read entire content of a file. */
std::string read_text(const std::string& filename)
{
    std::fstream f {filename, std::ios::in};
    if (!f) {
        std::println("Error opening file!");
        exit(1);
    }
    f.seekg(0, std::ios::end);
    auto size = f.tellg();
    f.seekg(0, std::ios::beg);
    std::string str(size, '\0');
    f.read(str.data(), size);
    return str;
}

/* Remove punctuation and change to lowercase. */
void process_text_inplace(std::string& text)
{
    // For each character, if it's punctuation replace with a space, otherwise lowercase it.
    std::ranges::for_each(text, [](char& c) {
        const std::string punctuation = "!\"&'()[]-,.:;?";
        if (punctuation.contains(c)) {
            c = ' ';
            return;
        }
        c = std::tolower(c);
    });
}

using dict = std::unordered_map<std::string, int>;

/* Count the number of times each word appears in input. */
dict count_appearances(const std::string& text)
{
    std::stringstream buffer {text};
    dict word_counts;
    std::string word;
    while (buffer >> word)
        word_counts[word]++; // The behaviour here is very different to Python! (zero initialization)
    return word_counts;
}

using pair = std::pair<std::string, int>;

/* Create a sorted vector from a map based on an item's value. */
std::vector<pair> sorted_word_list(const dict& word_counts)
{
    std::vector<pair> word_counts_sorted {begin(word_counts), end(word_counts)}; // auto word_counts_sorted = std::ranges::to<vector<pair>>(word_counts); // This is part of C++23 but doesn't work in GCC 13.2
    std::ranges::sort(word_counts_sorted, [](const auto a, const auto b){ return a.second > b.second; });
    return word_counts_sorted;
}

int main()
{
    auto text = read_text("data/shakespeare.txt");
    process_text_inplace(text);
    const auto word_counts = count_appearances(text);
    auto word_counts_sorted = sorted_word_list(word_counts);

    for (const auto& [word, count] : (word_counts_sorted | std::views::take(20))) {
        std::println("{:12s}{: 6d}", word, count);
    }
}